SET NOCOUNT ON;
DBCC DROPCLEANBUFFERS;
DBCC FREEPROCCACHE;


-- Initialize
DECLARE @ii INT;
DECLARE @IterationsToDo INT = 20000;
DECLARE @id VARCHAR(8);

DECLARE @log TABLE
  (TheOrder INT IDENTITY(1, 1),
WhatHappened VARCHAR(200),
WHENItDid DATETIME2 DEFAULT GetDate());

CREATE TABLE #names1 (name VARCHAR(80));
CREATE TABLE #names2 (name VARCHAR(80));
CREATE TABLE #names3 (name VARCHAR(80));
CREATE TABLE #names4 (name VARCHAR(80));
CREATE TABLE #names5 (name VARCHAR(80));

----start of timing
INSERT INTO @log(WhatHappened) SELECT 'Starting the test run';

--place at the start

/*Iteration with parameterized dynamic SQL*/
DBCC DROPCLEANBUFFERS;

DBCC FREEPROCCACHE;

SELECT @ii = 1;

WHILE @ii <= @IterationsToDo
  BEGIN
    SELECT @ii = @ii + 1;

    EXEC sp_executesql N'
  insert into #names1 (name) Select Title + '' ''+Firstname+ '' ''+Lastname 
  from person 
  where person_ID=@id', N'@id int', @id = @ii;
  END;

SELECT Count(*) FROM #names1;

INSERT INTO @log(WhatHappened)
  SELECT 'Ran ' + Convert(VARCHAR(5), @IterationsToDo)
         + ' ExecuteSQL queries';

/*Iteration with a prepared statement*/
DBCC DROPCLEANBUFFERS;
DBCC FREEPROCCACHE;

DECLARE @P1 INT;

SELECT @ii = 1;

EXEC sp_prepare @P1 OUTPUT, N'@Id INT',
  N'insert into #names2 (name) Select Title + '' ''+Firstname+ '' ''+Lastname 
from person where person_ID=@Id';

WHILE @ii <= @IterationsToDo
  BEGIN
    SELECT @ii = @ii + 1;

    EXEC sp_execute @P1, @ii;
  END;

EXEC sp_unprepare @P1;

SELECT Count(*) FROM #names2;

INSERT INTO @log(WhatHappened)
  SELECT 'Ran ' + Convert(VARCHAR(5), @IterationsToDo)
         + ' prepared queries';

/*Iteration with a 'raw' ad-hoc query*/
DBCC DROPCLEANBUFFERS;
DBCC FREEPROCCACHE;


SELECT @ii = 1;

WHILE @ii <= @IterationsToDo
  BEGIN
    SELECT @ii = @ii + 1, @id = Convert(VARCHAR(5), @ii);

    INSERT INTO #names3(name)
      SELECT Title + ' ' + FirstName + ' ' + LastName FROM Person WHERE Person_id = @id;
  END;

SELECT Count(*) FROM #names3;

INSERT INTO @log(WhatHappened)
  SELECT 'Ran ' + Convert(VARCHAR(5), @IterationsToDo)
         + ' Ad hoc raw queries';

/*The set-based approach*/
DBCC DROPCLEANBUFFERS;
DBCC FREEPROCCACHE;


INSERT INTO dbo.#names4(name) SELECT Title + ' ' + FirstName + ' '
                                    + LastName FROM dbo.Person;

SELECT Count(*) FROM #names4;

INSERT INTO @log(WhatHappened)
  SELECT 'Ran ' + Convert(VARCHAR(5), @IterationsToDo)
         + ' set-based SELECT INTO'; --

/*Iteration with a dynamic ad-hoc query*/
DBCC DROPCLEANBUFFERS;
DBCC FREEPROCCACHE;

SELECT @ii = 1;

WHILE @ii <= @IterationsToDo
  BEGIN
    SELECT @ii = @ii + 1, @id = Convert(VARCHAR(5), @ii);

    EXECUTE('insert into #names5 (name) Select Title + '' ''+Firstname+ '' ''+Lastname from person where person_ID=' + @id);
  END;

SELECT Count(*) FROM #names5;

INSERT INTO @log(WhatHappened)
  SELECT 'Ran ' + Convert(VARCHAR(5), @IterationsToDo)
         + ' Adhoc dynamic strings';




SELECT ending.WhatHappened,
  DateDiff(ms, starting.WHENItDid, ending.WHENItDid)
  FROM @log AS starting
    INNER JOIN @log AS ending
      ON ending.TheOrder = starting.TheOrder + 1;
--list out all the timings

--clear up
DROP TABLE #names1
DROP TABLE #names2
DROP TABLE #names3
DROP TABLE #names4
DROP TABLE #names5